<?php

namespace App\Model;

use Nette,
	Nette\Utils\Strings,
	Nette\Utils\ArrayHash,
	App\Model\EventManager;


/**
 * Layouts management.
 */
class LayoutManager extends Nette\Object
{
	const
		SEATS_TABLE_NAME = 'seats',
		SEATS_ID = 'id_seats',
		SEATS_S_LAYOUT_ID = 'seats_seats_layout_id',
		SEATS_AVAILABLE = 'seats_available',
		SEATS_NAME = 'seats_name',
		SEATS_PRICE = 'seats_price',
		
		ZONES_GROUP_TABLE_NAME = 'zones_group',
		ZONES_GROUP_ID = 'id_zones_group',
		ZONES_GROUP_IS_ACTIVE = 'zones_group_is_active',
		
		ZONES_TABLE_NAME = 'zones',
		ZONES_ID = 'id_zones',
		ZONES_NAME = 'zones_name',
		ZONES_ZONES_GROUP_ID = 'zones_zones_group_id',
		ZONES_PRICE = 'zones_price',
		ZONES_COORDS = 'zones_coords',
		ZONES_ROW_SEATS_NUM = 'zones_row_seats_num',
		
		SEATS_LAYOUT_TABLE_NAME = 'seats_layout',
		SEATS_LAYOUT_ID = 'id_seats_layout',
		SEATS_LAYOUT_NAME = 'seats_layout_name',
		SEATS_LAYOUT_FILENAME = 'seats_layout_filename',
		SEATS_LAYOUT_IS_ACTIVE = 'seats_layout_is_active',
		
		ZONES_SEATS_TABLE_NAME = 'zones_seats',
		ZONES_SEATS_ID = 'id_zones_seats',
		ZONES_SEATS_ZONES_ID = 'zones_seats_zones_id',
		ZONES_SEATS_SEATS_ID = 'zones_seats_seats_id',
		
		TICKETS_TEMPLATE_TABLE_NAME = 'tickets_template',
		TICKETS_TEMPLATE_ID = 'id_tickets_template',
		TICKETS_TEMPLATE_NAME = 'tickets_template_name',
		TICKETS_TEMPLATE_CODE = 'tickets_template_code',
		TICKETS_TEMPLATE_IS_ACTIVE = 'tickets_template_is_active';
		


	/** @var Nette\Database\Context */
	private $database;

	/** @var EventManager */
	private $eventManager;


	public function __construct(Nette\Database\Context $database, EventManager $eventManager)
	{
		$this->database = $database;
		$this->eventManager = $eventManager;
	}


 	/**
	 * Saves new layout image.
	 * @param string
	 * @param string
	 * @throws Exception
	 */
	public function addLayout($name, $filename)
	{
		try {
			$this->database->table(self::SEATS_LAYOUT_TABLE_NAME)->insert(array(
				self::SEATS_LAYOUT_NAME => $name,
				self::SEATS_LAYOUT_FILENAME => $filename
			));	
		} catch (\Exception $e) {
			throw new \Exception("Something went wrong. Please, try it again.");
		}	
	}


	/**
	 * Edits layout image.
	 * @param string
	 * @param string
	 * @param int
	 * @throws Exception
	 */
	public function editLayout($name, $filename, $id)
	{
		try {
			$lay = $this->database->table(self::SEATS_LAYOUT_TABLE_NAME)->get($id);
			if ($lay->seats_layout_name != $name) {
				$this->database->table(self::SEATS_LAYOUT_TABLE_NAME)->get($id)->update(array(
					self::SEATS_LAYOUT_NAME => $name,
					self::SEATS_LAYOUT_FILENAME => $filename
				));	
			}
		} catch (Nette\Database\UniqueConstraintViolationException $e) {
			throw new \Exception('The name '.$name.' already exists.');	
		} catch (\Exception $e) {
			throw new \Exception("Something went wrong. Please, try it again.");
		}	
	}


	/**
	 * Controls layout's duplicates.
	 * @param string
	 * @return int
	 * @throws Exception
	 */
	public function controlLayout($name)
	{
		try {
			$duplicate = $this->database->table(self::SEATS_LAYOUT_TABLE_NAME)->where(self::SEATS_LAYOUT_IS_ACTIVE, 1)->where(self::SEATS_LAYOUT_NAME, $name)->count('*');
			
			return $duplicate;
		} catch (\Exception $e) {
			throw new \Exception($e);
		}
			
	}


	/**
	 * Adds new zones to layout.
	 * @param  string
	 * @return void
	 * @throws DuplicateNameException
	 */
	public function addZones($jsonStr)
	{
		$json = json_decode($jsonStr, TRUE);
	   
		try {
			$layout_id = $this->database->table(self::SEATS_LAYOUT_TABLE_NAME)->max(self::SEATS_LAYOUT_ID);
			$zones_group_id = $this->database->table(self::ZONES_GROUP_TABLE_NAME)->insert(array(
				self::ZONES_GROUP_IS_ACTIVE => 1
			));		
			
			foreach ($json as $zonename => $value) {
		        $coords = $value['coords'];
				$price = $value['price'];
				
				$zones_id = $this->database->table(self::ZONES_TABLE_NAME)->insert(array(
					self::ZONES_ZONES_GROUP_ID => $zones_group_id->id_zones_group,
					self::ZONES_NAME => $zonename,
					self::ZONES_COORDS => $coords,
					self::ZONES_PRICE => $price,
					self::ZONES_ROW_SEATS_NUM => $value['number']
				));
				
				$seats = [];
				foreach ($value['seats'] as $row) {
					foreach ($row as $seat) {
						$seat['seats_seats_layout_id'] = $layout_id;
						array_push($seats, $seat);
					}
				}

				$seats_id = $this->database->table(self::SEATS_TABLE_NAME)->insert($seats);	
				$max_id = $seats_id->id_seats + count($seats) - 1;

				$zones_seats = [];

				for ($i = $seats_id->id_seats; $i <= $max_id; $i++) { 
					array_push($zones_seats, ['zones_seats_seats_id' => $i, 'zones_seats_zones_id' => $zones_id->id_zones]);
				}
				
				$this->database->table(self::ZONES_SEATS_TABLE_NAME)->insert($zones_seats);	
			}
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}


	/**
	 * Edits layout - deletes old data and updatea with new.
	 * @param  int
	 * @return void
	 * @throws Exception
	 */
	public function editZones($jsonStr, $id)
	{
		
		$json = json_decode($jsonStr, TRUE);
	   
		try {
			// DELETE
			$zones = $this->database->query("
				SELECT id_zones, zones_zones_group_id
				FROM zones
				JOIN zones_seats ON zones_seats.zones_seats_zones_id = zones.id_zones
				JOIN seats ON seats.id_seats = zones_seats.zones_seats_seats_id
				WHERE seats_seats_layout_id = ?
				GROUP BY zones_zones_group_id
			", $id)->fetchAll();

			$this->database->query("
				DELETE z 
				FROM zones_seats z
				JOIN seats ON z.zones_seats_seats_id = seats.id_seats
				JOIN zones ON z.zones_seats_zones_id = zones.id_zones 
				WHERE seats_seats_layout_id = ?
			", $id);

			$this->database->query("
				DELETE FROM seats
				WHERE seats_seats_layout_id = ?
			", $id);

			foreach ($zones as $zone) {
				$this->database->table(self::ZONES_TABLE_NAME)->where(self::ZONES_ID, $zone['id_zones'])->delete();
				$this->database->table(self::ZONES_GROUP_TABLE_NAME)->where(self::ZONES_GROUP_ID, $zone['zones_zones_group_id'])->delete();
			}



			// NEW
			$zones_group_id = $this->database->table(self::ZONES_GROUP_TABLE_NAME)->insert(array(
				self::ZONES_GROUP_IS_ACTIVE => 1
			));			
			
			foreach ($json as $zonename => $value) {
		        $coords = $value['coords'];
				$price = $value['price'];
				$seats = $value['seats'];
				
				$zones_id = $this->database->table(self::ZONES_TABLE_NAME)->insert(array(
					self::ZONES_ZONES_GROUP_ID => $zones_group_id->id_zones_group,
					self::ZONES_NAME => $zonename,
					self::ZONES_COORDS => $coords,
					self::ZONES_PRICE => $price,
					self::ZONES_ROW_SEATS_NUM => $value['number']
				));
				
				$seats = [];
				foreach ($value['seats'] as $row) {
					foreach ($row as $seat) {
						$seat['seats_seats_layout_id'] = $id;
						array_push($seats, $seat);
					}
				}

				$seats_id = $this->database->table(self::SEATS_TABLE_NAME)->insert($seats);	
				$max_id = $seats_id->id_seats + count($seats) - 1;

				$zones_seats = [];

				for ($i = $seats_id->id_seats; $i <= $max_id; $i++) { 
					array_push($zones_seats, ['zones_seats_seats_id' => $i, 'zones_seats_zones_id' => $zones_id->id_zones]);
				}
				
				$this->database->table(self::ZONES_SEATS_TABLE_NAME)->insert($zones_seats);	
				
			}

			$events = $this->database->table('events')->where('events_seats_layout_id', $id)->fetchAll();
			foreach ($events as $event) {
				$this->eventManager->refactorSeats($id, $event->id_events);
			}
		} catch (\Exception $e) {
			throw new \Exception("Something went wrong. Please, try it again.");	
		}	
	}


	/**
	 * Deletes layout.
	 * @param  int
	 * @return void
	 * @throws Exception
	 */
	public function deleteLayout($id)
	{
		try {
			$this->database->table(self::SEATS_LAYOUT_TABLE_NAME)->get($id)->update(array(
				self::SEATS_LAYOUT_IS_ACTIVE => "0"
			));	
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}
	
	
	/**
	 * Adds template.
	 * @param  array
	 * @return void
	 * @throws Exception
	 */
	public function addTemplate($temp)
	{
		try {
			$this->database->table(self::TICKETS_TEMPLATE_TABLE_NAME)->insert(array(
				self::TICKETS_TEMPLATE_NAME => $temp->name,
				self::TICKETS_TEMPLATE_CODE => $temp->code
			));	
		} catch (\Exception $e) {
			throw new \Exception("Something went wrong. Please, try it again.");	
		}
	}
	
	
	/**
	 * Gets template data for editing.
	 * @param  int
	 * @return array
	 * @throws Exception
	 */
	public function getTemplate($id)
	{
		try {
			$temp = $this->database->table(self::TICKETS_TEMPLATE_TABLE_NAME)->get($id);	
			return $temp;
		} catch (\Exception $e) {
			throw new \Exception("Something went wrong. Please, try it again.");	
		}
	}
	
	
	/**
	 * Edits template.
	 * @param  int
	 * @return void
	 * @throws Exception
	 */
	public function editTemplate($temp, $id)
	{
		try {
			$temp = $this->database->table(self::TICKETS_TEMPLATE_TABLE_NAME)->get($id)->update(array(
				self::TICKETS_TEMPLATE_CODE => $temp->code,
				self::TICKETS_TEMPLATE_NAME => $temp->name
			));	
		} catch (\Exception $e) {
			throw new \Exception("Something went wrong. Please, try it again.");	
		}
	}
	
	
	/**
	 * Deletes template.
	 * @param  int
	 * @return void
	 * @throws Exception
	 */
	public function deleteTemplate($id)
	{
		try {
			$this->database->table(self::TICKETS_TEMPLATE_TABLE_NAME)->get($id)->update(array(
				self::TICKETS_TEMPLATE_IS_ACTIVE => "0"
			));	
		} catch (\Exception $e) {
			throw new \Exception("Something went wrong. Please, try it again.");	
		}
	}
	
}