<?php

namespace App\Model;

use Nette,
	Nette\Utils\Strings,
	Nette\Utils\ArrayHash,
	Nette\Security\Passwords,
	Nette\Security\User;


/**
 * Users management.
 */
class UserManager extends Nette\Object implements Nette\Security\IAuthenticator
{
	const
		USERS_TABLE_NAME = 'users',
		USERS_ID = 'id_users',
		USERS_NAME = 'users_name',
		USERS_DESCRIPTION = 'users_description',
		USERS_IS_ACTIVE = 'users_is_active',
		USERS_ROLE = 'users_roles_id',
		USERS_PASSWORD_HASH = 'users_password',
		USERS_DEALER = 'users_dealers_id',
		USERS_EMAIL = 'users_email',
		USERS_ADDRESS = 'users_address_id',
        USERS_IS_REMOVED = 'users_is_removed',
		
		ROLES_TABLE_NAME = 'roles',
		ROLES_ID = 'id_roles',
		ROLES_NAME = 'roles_name',
		ROLES_PERMISSIONS = 'roles_permissions',
		
		DEALERS_TABLE_NAME = 'dealers',
		DEALERS_ID = 'id_dealers',
		DEALERS_NAME = 'dealers_name',
		DEALERS_IS_ACTIVE = 'dealers_is_active';


	/** @var Nette\Database\Context */
	private $database;

	/** @var Nette\Security\User */
	private $user;


	public function __construct(Nette\Database\Context $database)
	{
		$this->database = $database;
	}


	/**
	 * Performs an authentication.
	 * @return Nette\Security\Identity
	 * @throws Nette\Security\AuthenticationException
	 */
	public function authenticate(array $credentials)
	{
		list($username, $password) = $credentials;

		$row = $this->database->query("
				SELECT users.*, dealers_is_active
				FROM users
				JOIN dealers ON users_dealers_id = id_dealers
				WHERE users_name = ?
			", $username)->fetch();

		if (!$row || !Passwords::verify($password, $row[self::USERS_PASSWORD_HASH])) {
			throw new Nette\Security\AuthenticationException('Wrong username or password.', self::IDENTITY_NOT_FOUND);

		} elseif ($row->users_is_active === 0) {
			throw new Nette\Security\AuthenticationException('Your account was disabled.', self::INVALID_CREDENTIAL);

		} elseif ($row->dealers_is_active === 0) {
			throw new Nette\Security\AuthenticationException('Your dealer was disabled.', self::INVALID_CREDENTIAL);

		}elseif (Passwords::needsRehash($row[self::USERS_PASSWORD_HASH])) {
			$row->update(array(
				self::USERS_PASSWORD_HASH => Passwords::hash($password),
			));
		}

		$arr = $row;
		unset($arr[self::USERS_PASSWORD_HASH]);
		$role = $this->database->table(self::ROLES_TABLE_NAME)->where(self::ROLES_ID, $row[self::USERS_ROLE])->fetch();
		$credit = $this->database->table(self::DEALERS_TABLE_NAME)->where(self::DEALERS_ID, $row[self::USERS_DEALER])->select('dealers_credit')->fetch();
		$curr = $this->database->table('sale')->select('sale_currency')->fetch();
		$arr[self::ROLES_PERMISSIONS] = $role->roles_permissions;
		$arr['users_credit'] = $credit->dealers_credit;
		$arr['sale_currency'] = $curr->sale_currency;


		$this->database->table('logging')->insert(array(
			'logging_users_id' => $arr[self::USERS_ID],
			'logging_time' => date('Y-m-d h:i:s', time())
		));

		return new Nette\Security\Identity($row[self::USERS_ID], $role->roles_name, $arr);
	}


	/**
	 * Adds new user.
	 * @param  array
	 * @return void
	 * @throws DuplicateNameException
	 */
	public function add($values)
	{
		try {
			if ($values->city != "" || $values->street != "" || $values->house_num != "") {
				$a = $this->database->table('address')->insert(array(
					'address_city' => $values->city,
					'address_street' => $values->street,
					'address_house_num' => $values->house_num
				));
				$address_id = $a->id_address;
			} else {
				$address_id = 1;
			}
			
			$this->database->table(self::USERS_TABLE_NAME)->insert(array(
				self::USERS_NAME => $values->username,
				self::USERS_DESCRIPTION => $values->description,
				self::USERS_IS_ACTIVE => $values->is_active,
				self::USERS_ROLE => $values->role,
				self::USERS_EMAIL => $values->email,
				self::USERS_PASSWORD_HASH => Passwords::hash($values->password),
				self::USERS_DEALER => $values->dealer,
				self::USERS_ADDRESS => $address_id
			));	
			
		} catch (Nette\Database\UniqueConstraintViolationException $e) {
			throw new \Exception('The username '.$values->username.' already exists.');		
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}
	
	/**
	 * Updates user.
	 * @param  array
	 * @param  int
	 * @return void
	 * @throws Exception
	 */
	public function edit($values, $id)
	{
		try {			
			$u = $this->database->table(self::USERS_TABLE_NAME)->get($id);

			if ($values['city'] != "" || $values['street'] != "" || $values['house_num'] != "") {
				$this->database->table('address')->where('id_address', $u->users_address_id)->update(array(
					'address_city' => $values['city'],
					'address_street' => $values['street'],
					'address_house_num' => $values['house_num']
				));
				$address_id = $u->users_address_id;
			} else {
				$address_id = 1;
			}

			$this->database->table(self::USERS_TABLE_NAME)->get($id)->update(array(				 
				self::USERS_NAME => $values['username'], 
				self::USERS_DESCRIPTION => $values['description'],
				self::USERS_IS_ACTIVE => $values['is_active'],
				self::USERS_ROLE => $values['role'],
				self::USERS_EMAIL => $values['email'],
				self::USERS_DEALER => $values['dealer'],
				self::USERS_ADDRESS => $address_id
			));
			
			
			if ($values['password'] != "") {
				$this->database->table(self::USERS_TABLE_NAME)->get($id)->update(array(self::USERS_PASSWORD_HASH => Passwords::hash($values['password'])));
			}	
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}
	
	
	/**
	 * Deletes user.
	 * @param  int
	 * @return void
	 * @throws Exception
	 */
	public function delete($id)
	{
		try {
			$this->database->table(self::USERS_TABLE_NAME)->get($id)->update(array(
				self::USERS_IS_ACTIVE => "0",
                self::USERS_IS_REMOVED => "1"
			));	
		} catch (\Exception $e) {
			throw new \Exception($e);	
		}
	}
	
	/**
	 * Selects all roles for select input in UserForm.
	 * @return array $roles
	 */
	public function getRoles()
	{
		$roles = $this->database->table(self::ROLES_TABLE_NAME)->select(self::ROLES_NAME.','.self::ROLES_ID)->fetchPairs(self::ROLES_ID, self::ROLES_NAME);		
		
		return $roles;
	}
	
	
	/**
	 * Selects user's address
	 */
	public function getAddress($id)
	{
		$user = $this->database->table(self::USERS_TABLE_NAME)->where(self::USERS_ID, $id)->select(self::USERS_ADDRESS)->fetch();
		$address = $this->database->table('address')->get($user->users_address);
		
		return $address;
	}
	
	
	/**
	 * Selects all dealers for select input in UserForm.
	 * @return array $dealers
	 */
	public function getDealers()
	{
		$dealers = $this->database->table(self::DEALERS_TABLE_NAME)->where(self::DEALERS_IS_ACTIVE, 1)->select(self::DEALERS_NAME.','.self::DEALERS_ID)->fetchPairs(self::DEALERS_ID, self::DEALERS_NAME);		
		
		return $dealers;
	}
	
	
	/**
	 * Selects all user's information for update.
	 * @param array
	 * @return array $user_data
	 */
	public function getData($id)
	{			
		$user_data = $this->database->table(self::USERS_TABLE_NAME)->where(self::USERS_ID, $id)->fetch();	
		
		return $user_data; 
	}
	


}